/*****************************************************************************
*
* hsTimer.cpp   
* Copyright 1999, MindWare
*
* - Implementation of hsTimer class, see hsTimer.h
*****************************************************************************/

#include "hstimer.h"

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #include <windows.h>
#else
  #include <time.h>           // for nanosleep
  #include <sys/time.h>       // For gettimeofday
  #include <math.h>           // for fmod
#endif

namespace MobyDisk {

// Number of ticks per millisecond is set during call init()
hsTime hsTimer::ticksPerMS;

////////////////////////////////////////////////////////////////////////////////
// Initialize the timer
// - This must be called first!
//
////////////////////////////////////////////////////////////////////////////////
void hsTimer::init()
{
  #ifdef WIN32
      LARGE_INTEGER frequency;
      QueryPerformanceFrequency(&frequency);    // Get timer accuracy
      ticksPerMS = frequency.QuadPart/1000;     // Compute ticks/ms
   #else
      ticksPerMS = 1000;                        // gettimeofday returns microsecond precision
   #endif
}

////////////////////////////////////////////////////////////////////////////////
// Return the current time
//
// RET:  Time as an hsTime.  This value alone is meaningless until you
//       pass it to getElapsed.
////////////////////////////////////////////////////////////////////////////////
hsTime hsTimer::getTime()
{
   #ifdef WIN32
      LARGE_INTEGER now;
      QueryPerformanceCounter(&now);
      return now.QuadPart;
   #else
      timeval tv;
      gettimeofday(&tv,NULL);
      return tv.tv_sec * 1000000 + tv.tv_usec;
   #endif
}

////////////////////////////////////////////////////////////////////////////////
// Return elapsed time in milliseconds
//
// ARGS: start;   Start time as an hsTime
//       stop;    Stop time as an hsTime.  If it is not specified, it is now
// RET:  Milliseconds elapsed between start and stop
////////////////////////////////////////////////////////////////////////////////
double hsTimer::getElapsed(hsTime start, hsTime stop)
{
   if (stop==-1)
      stop = getTime();
   return static_cast<double>(stop-start) / ticksPerMS;
}

////////////////////////////////////////////////////////////////////////////////
// Wait the specified number of milliseconds, as accurately as possible
// - Spin loop: This is not nice, and it does not yield the CPU
//
// ARGS: millisecs;  Number of milliseconds to wait
////////////////////////////////////////////////////////////////////////////////
void hsTimer::hsDelay(double millisecs)
{
   hsTime start = getTime();
   while ( getElapsed(start) < millisecs ) ;
}

////////////////////////////////////////////////////////////////////////////////
// Wait the specified number of milliseconds, less accurate but nicer
// - Spin loop: This will yield during the loop and may not be accurate
//              Even if passed a time of zero this will yield at least once
//
// ARGS: millisecs; Number of milliseconds to wait
////////////////////////////////////////////////////////////////////////////////
void hsTimer::niceDelay(double millisecs)
{
   // Grab the start time
   hsTime start = getTime();
   
   // Set the remaining time to sleep
   // - This is updated in each iteration of the loop
   double dRemaining = millisecs;

   // Put this in a loop because we may not sleep long enough
   do
   {
      // Either of these sleep functions make wake up early, so put
      // them in a loop
      #ifdef WIN32
         // Yields the timeslice for at least the specified time
         // - accurate to milliseconds, so we truncate
         Sleep(static_cast<DWORD>(dRemaining));
      #else
         // nanosleep will suspend execution for an interval
         // - It takes numbers up to a nanoseconds
         // - It is probably only accurate to milliseconds though
         timespec requested;
         requested.tv_sec = static_cast<time_t>(millisecs / 1000);
         requested.tv_nsec = static_cast<long>(fmod(millisecs,1000) * 1000000);
         nanosleep(&requested,NULL);
      #endif

      // At the end of this loop, we subtract off the actual time we slept
   } while ( (dRemaining = millisecs - getElapsed(start)) > 0 );
}

} // namespace MobyDisk
